"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const mkdirp_1 = __importDefault(require("mkdirp"));
const loader_utils_1 = require("loader-utils");
const path_1 = __importDefault(require("path"));
const core_1 = require("@graphql-codegen/core");
const typescriptPlugin = __importStar(require("@graphql-codegen/typescript"));
const typescriptOperationsPlugin = __importStar(require("@graphql-codegen/typescript-operations"));
const typescriptReactApolloPlugin = __importStar(require("@graphql-codegen/typescript-react-apollo"));
const gen_dts_1 = __importDefault(require("./gen-dts"));
const graphql_1 = require("graphql");
const graphql_toolkit_1 = require("graphql-toolkit");
const { writeFile } = fs_1.promises;
const libDir = path_1.default.resolve(__dirname, '..');
const tsxBaseDir = path_1.default.join(libDir, '__generated__');
mkdirp_1.default.sync(tsxBaseDir);
const defaultCodegenConfig = {
    config: {
        // withHOC: false,  // True by default
        withHooks: true,
    },
    plugins: [
        { typescript: {} },
        { 'typescript-operations': {} },
        { 'typescript-react-apollo': {} },
    ],
    pluginMap: {
        typescript: typescriptPlugin,
        'typescript-operations': typescriptOperationsPlugin,
        'typescript-react-apollo': typescriptReactApolloPlugin,
    },
};
const graphlqCodegenLoader = async function (gqlContent) {
    const options = loader_utils_1.getOptions(this);
    const callback = this.async();
    const { resourcePath: gqlFullPath, rootContext: userDir } = this;
    const gqlRelPath = path_1.default.relative(userDir, gqlFullPath);
    const tsxRelPath = gqlRelPath + `.tsx`;
    const tsxFullPath = path_1.default.join(tsxBaseDir, tsxRelPath);
    const dtsFullPath = `${gqlFullPath}.d.ts`;
    // Pretend .tsx for later loaders.
    // babel-loader at least doesn't respond the .graphql extension.
    this.resourcePath = `${gqlFullPath}.tsx`;
    // TODO: Memoize
    const loadedSchema = await graphql_toolkit_1.loadSchema(options.schema);
    const schema = graphql_1.parse(graphql_1.printSchema(loadedSchema));
    const codegenConfig = {
        ...defaultCodegenConfig,
        ...options,
        schema,
        filename: tsxFullPath,
        documents: [
            {
                filePath: gqlFullPath,
                content: graphql_1.parse(gqlContent),
            }
        ],
    };
    const tsxContent = await core_1.codegen(codegenConfig);
    const dtsContent = await gen_dts_1.default(tsxFullPath);
    await writeFile(dtsFullPath, dtsContent);
    callback(undefined, tsxContent);
};
exports.default = graphlqCodegenLoader;
